#!/bin/sh
#
##
# clpgcdns-watch.sh
##
# monitoring the clpgcdns
##


echo "clpgcdns watch script start."


##
# success & error code
##
GCDNSW_SUCCESS=0					# succeed
GCDNSW_ERR_GET_VALUE=1				# failed to get the setting value
GCDNSW_ERR_GET_ZONENAME=2			# get zone name has failed
GCDNSW_ERR_GET_RECORDNAME=3			# get record name has failed
GCDNSW_ERR_GET_RECORDTYPE=4			# get record type has failed
GCDNSW_ERR_GET_RECORDTTL=5			# get ttl has failed
GCDNSW_ERR_GET_RECORDIP=6			# get ip address has failed
GCDNSW_ERR_GET_RECORD=7				# get dns record has failed
GCDNSW_ERR_NO_RECORD=8				# there is no designated dns record


##
# set value
##
TARGET_RSC="${CLP_OCF_PARAM1}"			# resource name
LOGCMD_ID=3601							# event id of clplogcmd
SRV_FULLNAME="`uname -n`"				# get hostname (use output file path)
SRV_NAME="`echo ${SRV_FULLNAME%%.*}`"	# delete domain name

if [ -z "$TARGET_RSC" ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_VALUE}] gcdnsw: Failed to obtain the setting value." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "failed to get the setting value." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_VALUE}
fi

##
# get target resource config
##
ZONE_NAME="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/zone_name -p gcdns`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_ZONENAME}] gcdnsw: Failed to obtain the zone name." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "$ZONE_NAME" | head -n 1 1>&2
	echo "get zone name has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_ZONENAME}
fi

RECORD_NAME="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/dns_name -p gcdns`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_RECORDNAME}] gcdnsw: Failed to obtain the DNS name." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "$RECORD_NAME" | head -n 1 1>&2
	echo "get record name has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDNAME}
fi

RECORD_TYPE="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_type -p gcdns`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_RECORDTYPE}] gcdnsw: Failed to obtain the record type." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "$RECORD_TYPE" | head -n 1 1>&2
	echo "get record type has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDTYPE}
fi

RECORD_TTL="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_ttl -p gcdns`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_RECORDTTL}] gcdnsw: Failed to obtain the TTL." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "$RECORD_TTL" | head -n 1 1>&2
	echo "get ttl has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDTTL}
fi

RECORD_IP="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_ip -p gcdns -s \"${SRV_NAME}\"`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_RECORDIP}] gcdnsw: Failed to obtain the IP address." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "$RECORD_IP" | head -n 1 1>&2
	echo "get ip address has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDIP}
fi

echo "get config has succeeded."


##
# get dns record
##
CURRENT="`gcloud dns record-sets list -z=\"${ZONE_NAME}\" --name=\"${RECORD_NAME}\" --type=\"${RECORD_TYPE}\" \
	--format=\"table[no-heading](name,type,ttl, rrdatas.list():label=DATA)\"`"
if [ $? -ne 0 ]
then
	clplogcmd -m "[${GCDNSW_ERR_GET_RECORD}] gcdnsw: Failed to obtain the record set." --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "get dns record has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORD}
elif [ -z "$CURRENT" ]
then
	clplogcmd -m "[${GCDNSW_ERR_NO_RECORD}] gcdnsw: No record set to be monitored.(get dns record)" --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "there is no designated dns record." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD}
fi


##
# ttl ip check
##
CURRENT_TTL="`echo ${CURRENT} | awk '{print $3}'`"
CURRENT_IP="`echo ${CURRENT} | awk '{print $4}'`"

if [ "$CURRENT_TTL" -ne "$RECORD_TTL" ]
then
	clplogcmd -m "[${GCDNSW_ERR_NO_RECORD}] gcdnsw: No record set to be monitored.(ttl check)" --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "CURRENT_TTL = ${CURRENT_TTL} / RECORD_TTL = ${RECORD_TTL}" 1>&2
	echo "ttl is not suitable." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD}
fi

if [ "$CURRENT_IP" != "$RECORD_IP" ]
then
	clplogcmd -m "[${GCDNSW_ERR_NO_RECORD}] gcdnsw: No record set to be monitored.(ip check)" --alert --syslog -l ERR -i ${LOGCMD_ID} > /dev/null 2>&1
	echo "CURRENT_IP = ${CURRENT_IP} / RECORD_IP = ${RECORD_IP}" 1>&2
	echo "ip address is not suitable." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD}
fi

##
# end
##
echo "clpgcdns watch script has succeeded."

exit ${GCDNSW_SUCCESS}

